using CairoMakie
using CairoMakie.Makie.Colors
using Printf
using TOML


include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot_u_mu(rootdir::String, dirs::Vector)

    for dir in dirs
        @assert isdir(joinpath(rootdir,dir)) "missing output directory $(joinpath(rootdir,dir))"
    end

    ### load
    data = [ readdata(joinpath(rootdir, dir), ["u","smoothed_mu"],
                      output="output1d.h5") for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml")) for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    fv_data = readdata(joinpath(rootdir, "fv_muscl_1024"), ["u"],
                       output="output1d.h5")

    ### sort wrt k
    P          = sortperm(ks)
    data       = data[P]
    pars       = pars[P]
    ns         = ns[P]
    ks         = ks[P]
    dirs       = dirs[P]

    ts = [ d.t for d in data ]
    xs = [ d.x for d in data ]
    us = [ d.u for d in data ]
    μs = [ d.smoothed_mu for d in data ]

    idxs_tend = length.(ts)
    idx_tend = minimum(idxs_tend)

    stack_μs   = stack.(μs)
    @show minimum(minimum, stack_μs)
    @show maximum(maximum, stack_μs)
    min_μ, max_μ = 0.0, 0.004
    Δμ = max_μ-min_μ
    yticks_μ = range(min_μ, max_μ, length=6)

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1200,1000))
    gl_u = fig[1,1:2] = GridLayout()
    gl_μs = fig[2,1:2] = GridLayout()
    # gl_cb = fig[2,3] = GridLayout(; alignmode=Outside())

    ax_u = Axis(gl_u[1,1], xlabel=L"x")
    ax_μs = [ Axis(gl_μs[1,i], xlabel=L"x") for i in 1:length(dirs) ]

    rng = 1:20:1024
    scatter!(ax_u, fv_data.x[rng], fv_data.u[idx_tend][rng],
           color=colorant"deeppink", label="FV, K = 1024",
           markersize=15)
    ylabel_u=L"u(x)"
    Label(fig[1,0], ylabel_u,
          tellwidth=true, tellheight=false, rotation=pi/2, halign=:left)
    for (i,dir) in enumerate(dirs)
        lbl = "K = $(ks[i])"
        for ax in (ax_u,)
            lines!(ax, xs[i], us[i][idx_tend #= final time =#], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
        end
    end

    xlims!(ax_u, (-0.05,1.05))
    ylims!(ax_u, (-1.1,1.1))
    ax_u.xticks[] = range(0.0,1.0,step=0.25)
    # ax_u.yticks[] = range(1.0,1.75,step=0.2)

    # plot the heatmaps for the viscosity
    ylabel_μ=L"\mu(t,x)"
    Label(fig[4,1:2], ylabel_μ,
          tellwidth=false, tellheight=true)
    Label(fig[2,0], L"t",
          tellwidth=true, tellheight=false, rotation=pi/2)
    heatmaps = []
    for (i,dir) in enumerate(dirs)
        h = heatmap!(ax_μs[i], xs[i], ts[i], stack_μs[i], colormap=:turbo,
                     colorrange=(min_μ,max_μ))
        translate!(h, 0, 0, -100)
        push!(heatmaps, h)
    end
    white = RGBA(1,1,1,1)
    for ax in ax_μs
        xlims!(ax, (0.05,0.95))
        ax.xtickcolor[] = white
        ax.ytickcolor[] = white
        ax.xminortickcolor[] = white
        ax.yminortickcolor[] = white
        ax.xgridcolor[] = white
        ax.ygridcolor[] = white
        ax.xminorgridcolor[] = white
        ax.yminorgridcolor[] = white
    end
    for ax in ax_μs[2:end]
        hideydecorations!(ax, ticks=false, grid=false, minorticks=false, minorgrid=false)
        linkaxes!(ax_μs[1], ax)
    end

    gg = GridLayout(gl_u[1,1], tellwidth=false, tellheight=false,
                    halign=:left, valign=:top)
    box = Box(gg[1,1], color=:white, strokecolor=:black)
    tend = ts[1][idx_tend]
    str_t = @sprintf "%.2f" tend
    lbl = Label(gg[1,1], text=L"t = %$tend", padding=(30,30,10,10))
    translate!(box.blockscene, (0,0,100))
    translate!(lbl.blockscene, (0,0,100))

    cb = Colorbar(fig[3,1:2], heatmaps[end], vertical=false, alignmode=Outside(),
                  flipaxis=false, size=24, width=Relative(0.9))
    cb.ticks[] = yticks_μ
    Legend(gl_u[0,1], ax_u, colgap=40, orientation=:horizontal, width=nothing, tellwidth=false)
    rowgap!(gl_u, 1, 0)

    return fig

end


function make_plot_no_viscosity(rootdir::String, dirs::Vector; output::String="output1d.h5")

    for dir in dirs
        @assert isdir(joinpath(rootdir,dir)) "missing output directory $(joinpath(rootdir,dir))"
    end

    ### load
    data = [ readdata(joinpath(rootdir, dir), ["u"],
                      output=output) for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml")) for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    fv_data = readdata(joinpath(rootdir, "fv_muscl_1024"), ["u"],
                       output="output1d.h5")

    ### sort wrt k
    P          = sortperm(ks)
    data       = data[P]
    pars       = pars[P]
    ns         = ns[P]
    ks         = ks[P]
    dirs       = dirs[P]

    ts = [ d.t for d in data ]
    xs = [ d.x for d in data ]
    us = [ d.u for d in data ]

    idxs_tend = length.(ts)
    idx_tend = minimum(idxs_tend)
    idx_tend_fv = (allequal(idxs_tend) && idx_tend == length(first(ts))) ? length(fv_data.t) : idx_tend

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1200,400))
    gl_u = fig[1,1] = GridLayout()

    ax_u = Axis(gl_u[1,1], xlabel=L"x")

    rng = 1:20:1024
    scatter!(ax_u, fv_data.x[rng], fv_data.u[idx_tend_fv][rng],
           color=colorant"deeppink", label="FV, K = 1024",
           markersize=15)
    ylabel_u=L"u(x)"
    Label(fig[1,0], ylabel_u,
          tellwidth=true, tellheight=false, rotation=pi/2, halign=:left)
    for (i,dir) in enumerate(dirs)
        lbl = "K = $(ks[i])"
        for ax in (ax_u,)
            lines!(ax, xs[i], us[i][idx_tend #= final time =#], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
        end
    end

    gg = GridLayout(gl_u[1,1], tellwidth=false, tellheight=false,
                    halign=:left, valign=:top)
    box = Box(gg[1,1], color=:white, strokecolor=:black)
    tend = ts[1][idx_tend]
    str_t = @sprintf "%.2f" tend
    lbl = Label(gg[1,1], text=L"t = %$tend", padding=(30,30,10,10))
    translate!(box.blockscene, (0,0,100))
    translate!(lbl.blockscene, (0,0,100))

    xlims!(ax_u, (-0.05,1.05))
    ylims!(ax_u, (-1.1,1.1))
    ax_u.xticks[] = range(0.0,1.0,step=0.25)
    # ax_u.yticks[] = range(1.0,1.75,step=0.2)

    Legend(gl_u[0,1], ax_u, colgap=40, orientation=:horizontal, width=nothing, tellwidth=false)
    rowgap!(gl_u, 1, 0)

    return fig

end


#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))_u_mu"
fontsize = 40

for n in [4,5,6,7,8,9]
    dirs = [
             "avmda_$(n)_21_cmax_1",
             "avmda_$(n)_41_cmax_1",
             "avmda_$(n)_81_cmax_1",
             ]
    fig = with_theme(theme; fontsize) do
        make_plot_u_mu(rootdir, dirs)
    end
    save(joinpath(rootdir, "$(plotname_prefix)_avmda_n$(n)_cmax_1.png"), fig)

    dirs = [
             "avmda_$(n)_21_cmax_5",
             "avmda_$(n)_41_cmax_5",
             "avmda_$(n)_81_cmax_5",
             ]
    fig = with_theme(theme; fontsize) do
        make_plot_u_mu(rootdir, dirs)
    end
    save(joinpath(rootdir, "$(plotname_prefix)_avmda_n$(n)_cmax_5.png"), fig)

    dirs = [
             "avmda_$(n)_21_cmax_1",
             "avmda_$(n)_41_cmax_1",
             "avmda_$(n)_81_cmax_1",
             ]
    fig = with_theme(theme; fontsize) do
        make_plot_no_viscosity(rootdir, dirs, output="interpolate.h5")
    end
    save(joinpath(rootdir, "$(plotname_prefix)_avmda_n$(n)_cmax_1_interpolation.png"), fig)

    dirs = [
             "avmda_$(n)_21_cmax_5",
             "avmda_$(n)_41_cmax_5",
             "avmda_$(n)_81_cmax_5",
             ]
    fig = with_theme(theme; fontsize) do
        make_plot_no_viscosity(rootdir, dirs, output="interpolate.h5")
    end
    save(joinpath(rootdir, "$(plotname_prefix)_avmda_n$(n)_cmax_5_interpolation.png"), fig)
end

dirs = [
         "dg_5_21",
         "dg_5_41",
         "dg_5_81",
         ]
fig = with_theme(theme; fontsize) do
    make_plot_no_viscosity(rootdir, dirs)
end
save(joinpath(rootdir, "$(plotname_prefix)_dg_n5.png"), fig)

dirs = [
         "dg_4_21",
         "dg_4_41",
         "dg_4_81",
         ]
fig = with_theme(theme; fontsize) do
    make_plot_no_viscosity(rootdir, dirs)
end
save(joinpath(rootdir, "$(plotname_prefix)_dg_n4.png"), fig)

end
