using CairoMakie
using CairoMakie.Makie.Colors
using TOML


include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot(rootdir::String, dirs::Vector{String})

    for dir in dirs
        @assert isdir(joinpath(rootdir,dir)) "missing output directory $(joinpath(rootdir,dir))"
    end

    ### load
    data = [ readdata(joinpath(rootdir, dir), ["u"],
                      output="interpolate.h5") for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml")) for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    ### sort wrt k
    P          = sortperm(ks)
    data       = data[P]
    pars       = pars[P]
    ns         = ns[P]
    ks         = ks[P]
    dirs       = dirs[P]

    @assert allequal(ns)
    n = first(ns)

    ts = [ d.t for d in data ]
    xs = [ d.x for d in data ]
    us = [ d.u for d in data ]

    #######################################################################
    #                           self-convergence                          #
    #######################################################################

    dus = Any[]
    for i = 1:length(us)-1
        u1, u2 = us[i], us[i+1]
        du = [ uu1 .- uu2 for (uu1,uu2) in zip(u1,u2) ]
        push!(dus, du)
    end

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1400,500))
    gl_u = fig[1,1] = GridLayout()

    ylabel_u=L"\log_2(|u_1(x) - u_2(x)|)"
    ax_u = Axis(gl_u[1,1], xlabel=L"x", ylabel=ylabel_u)

    for (i,du) in enumerate(dus)
        k1, k2 = ks[i], ks[i+1]
        if i == 1
            lbl = L"|K_1 - K_2| = |%$k1 - %$k2|"
        else
            lbl = L"|%$k1 - %$k2|"
        end
        lines!(ax_u, xs[i], log2.(abs.(du[end #= final time =#])), label=lbl,
               color=colors[periodic_index(i,length(colors))])
        if i > 1
            s = (k2/k1)^(n)
            lines!(ax_u, xs[i], log2.(s*abs.(du[end #= final time =#])),
                   color=(colors[periodic_index(i-1,length(colors))],0.5),
                   linestyle=:dash)
        end
    end

    xlims!(ax_u, (-0.05,1.05))
    ylims!(ax_u, (-53,3))

    Legend(gl_u[0,1], ax_u, colgap=40, orientation=:horizontal, width=nothing, tellwidth=false)
    rowgap!(gl_u, 1, 0)

    return fig

end

#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))_u_mu"
fontsize = 40

for n in [4,5,6,7,8,9]
    dirs = [
             "avmda_$(n)_21_cmax_1",
             "avmda_$(n)_41_cmax_1",
             "avmda_$(n)_81_cmax_1",
             "avmda_$(n)_161_cmax_1",
             ]
    fig = with_theme(theme; fontsize) do
        make_plot(rootdir, dirs)
    end
    save(joinpath(rootdir, "$(plotname_prefix)_selfcvg_n$(n)_cmax_1.png"), fig)

    dirs = [
             "avmda_$(n)_21_cmax_5",
             "avmda_$(n)_41_cmax_5",
             "avmda_$(n)_81_cmax_5",
             "avmda_$(n)_161_cmax_5",
             ]
    fig = with_theme(theme; fontsize) do
        make_plot(rootdir, dirs)
    end
    save(joinpath(rootdir, "$(plotname_prefix)_selfcvg_n$(n)_cmax_5.png"), fig)
end

end
