using CairoMakie
using HDF5
using FFTW
using TOML


include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot(rootdir::String, dirs::Vector{String})

    for dir in dirs
        @assert isdir(joinpath(rootdir,dir)) "missing output directory $(joinpath(rootdir,dir))"
    end

    ### load
    data = [ readdata(joinpath(rootdir, dir), ["rho","smoothed_mu","p","v"],
                      output="output1d.h5") for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml")) for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    ### sort wrt k
    P          = sortperm(ks)
    data       = data[P]
    pars       = pars[P]
    ns         = ns[P]
    ks         = ks[P]
    dirs       = dirs[P]

    ts = [ d.t for d in data ]
    xs = [ d.x for d in data ]
    ρs = [ d.rho for d in data ]
    ps = [ d.p for d in data ]
    vs = [ d.v for d in data ]
    μs = [ d.smoothed_mu for d in data ]

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(900,800))
    gl_ρ = fig[1,1] = GridLayout()
    gl_p = fig[2,1] = GridLayout()
    gl_v = fig[3,1] = GridLayout()
    gl_μ = fig[4,1] = GridLayout()

    ax_ρ = Axis(gl_ρ[1,1], xlabel=L"x")
    ax_p = Axis(gl_p[1,1], xlabel=L"x")
    ax_v = Axis(gl_v[1,1], xlabel=L"x")
    ax_μ = Axis(gl_μ[1,1], xlabel=L"x")
    Label(fig[1,0], L"\rho(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(fig[2,0], L"p(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(fig[3,0], L"v(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(fig[4,0], L"\mu(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)

    for (i,dir) in enumerate(dirs)
        lbl = "K = $(ks[i])"
        lines!(ax_ρ, xs[i], ρs[i][end #= final time =#], label=lbl,
               color=colors[periodic_index(i,length(colors))])
        lines!(ax_p, xs[i], ps[i][end #= final time =#], label=lbl,
               color=colors[periodic_index(i,length(colors))])
        lines!(ax_v, xs[i], vs[i][end #= final time =#], label=lbl,
               color=colors[periodic_index(i,length(colors))])
        lines!(ax_μ, xs[i], μs[i][end #= final time =#], label=lbl,
               color=colors[periodic_index(i,length(colors))])
    end

    hidexdecorations!(ax_ρ, ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(ax_p, ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(ax_v, ticks=false, grid=false, minorgrid=false, minorticks=false)
    linkxaxes!(ax_ρ, ax_p, ax_v, ax_μ)
    Legend(fig.layout[0,1], ax_ρ, colgap=10, orientation=:horizontal, width=nothing, tellwidth=false,
           nbanks=1)
    rowgap!(fig.layout, 0)

    return fig

end

#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))_rho_mu"
display_plot = true

dirs = [
         "mono1_avmda",
         "mono2_avmda",
         "mono3_avmda",
         "mono4_avmda",
         ]
fig = with_theme(theme; fontsize=34) do
    make_plot(rootdir, dirs)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_mono_avmda.png"), fig)

end
