using CairoMakie
using TOML
using Printf


include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot(rootdir::String, dir::String, ts::Vector{Float64})

    ### load
    data = readdata(joinpath(rootdir, dir), ["rho","smoothed_mu","p","q"],
                    output="output1d.h5")
    pars = TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml"))
    n = pars["Mesh"]["n"]
    k = pars["Mesh"]["k"]

    idx_ts = [ something(findfirst(t -> t≈ti, data.t)) for ti in ts ]
    x = data.x
    ρs = [ data.rho[i] for i in idx_ts ]
    ps = [ data.p[i] for i in idx_ts ]
    vs = [ data.q[i]./data.rho[i] for i in idx_ts ]
    μs = [ data.smoothed_mu[i] for i in idx_ts ]

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(600,1000))
    gl_ρ = fig[1,1] = GridLayout()
    gl_p = fig[2,1] = GridLayout()
    gl_v = fig[3,1] = GridLayout()
    gl_μ = fig[4,1] = GridLayout()

    ax_ρ = Axis(gl_ρ[1,1], xlabel=L"x")
    ax_p = Axis(gl_p[1,1], xlabel=L"x")
    ax_v = Axis(gl_v[1,1], xlabel=L"x")
    ax_μ = Axis(gl_μ[1,1], xlabel=L"x")
    Label(fig[1,0], L"\rho(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(fig[2,0], L"p(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(fig[3,0], L"v(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(fig[4,0], L"\mu(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)

    for (i,t) in enumerate(ts)
        str_t = @sprintf "%0.2f" t
        lbl = L"t = %$str_t"
        lines!(ax_ρ, x, ρs[i], label=lbl,
               color=colors[periodic_index(i,length(colors))])
        lines!(ax_p, x, ps[i], label=lbl,
               color=colors[periodic_index(i,length(colors))])
        lines!(ax_v, x, vs[i], label=lbl,
               color=colors[periodic_index(i,length(colors))])
        lines!(ax_μ, x, μs[i], label=lbl,
               color=colors[periodic_index(i,length(colors))])
    end

    hidexdecorations!(ax_ρ, ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(ax_p, ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(ax_v, ticks=false, grid=false, minorgrid=false, minorticks=false)
    linkxaxes!(ax_ρ, ax_p, ax_v, ax_μ)
    Legend(fig.layout[0,1], ax_ρ, colgap=10, orientation=:horizontal, width=nothing, tellwidth=false,
           nbanks=2)
    rowgap!(fig.layout, 0)

    ylims!(ax_ρ, (0.05,1.05))
    ylims!(ax_p, (0.05,1.05))
    ylims!(ax_v, (-0.10,1.10))
    ylims!(ax_μ, (-0.001, 0.007))

    ax_ρ.yticks[] = 0.0:0.2:1.0
    ax_p.yticks[] = 0.0:0.2:1.0
    ax_v.yticks[] = 0.0:0.2:1.0
    ax_μ.yticks[] = 0.000:0.002:0.006

    return fig

end

#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))_rho_mu"
display_plot = true

ts = [ 0.0, 0.05, 0.15, 0.2, 0.25 ]
dirs = [
         "mono1_avmda",
         "mono2_avmda",
         "mono3_avmda",
         "mono4_avmda",
         ]
with_theme(theme; fontsize=30) do
    for dir in dirs
        fig = make_plot(rootdir, dir, ts)
        display_plot && display(fig)
        save(joinpath(rootdir, "$(plotname_prefix)_$(dir)_evo.png"), fig)
    end
end

end
