using CairoMakie
using CairoMakie.Makie.Colors # RGBA
using TOML
using Printf
using DataFrames

include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot(rootdir::String, dirs::Vector{String})

    display(dirs)

    for dir in dirs
        @assert isdir(joinpath(rootdir,dir)) "missing output directory $(joinpath(rootdir,dir))"
    end

    ### load
    data = [ readdata(joinpath(rootdir, dir), ["ρ_err_norm"],
                      output="output0d.h5") for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(dir).toml")) for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    df = DataFrame(:data => data, :n => ns, :k => ks)

    u_ns = unique(ns)
    u_ks = unique(ks)

    ymin, ymax = -42, 3
    yticks = -50:5:10

    #######################################################################
    #                              plotting                               #
    #######################################################################

    ncols = length(u_ns)
    fig = Figure(size=(1600,500))
    gls = [ fig[1,i] = GridLayout() for i in 1:ncols ]
    axs = [ Axis(gl[1,1], xlabel=L"t") for gl in gls ]

    for (n,gl) in zip(u_ns,gls)
        gg = GridLayout(gl[1,1], tellwidth=false, tellheight=false,
                        halign=:center, valign=:top)
        box = Box(gg[1,1], color=:white, strokecolor=:black)
        lbl = Label(gg[1,1], text=L"N = %$n", padding=(30,30,10,10))
        translate!(box.blockscene, (0,0,100))
        translate!(lbl.blockscene, (0,0,100))
    end

    for (n,ax) in zip(u_ns,axs)
        subdf = filter(row -> row.n==n, df)
        sort!(subdf, :k)
        for (i,row) in enumerate(eachrow(subdf))
            ρs = row.data.ρ_err_norm
            t = row.data.t
            ρ0 = ρs[1]
            abs_dρs = [ abs(ρi[1] - ρ0[1]) for ρi in ρs ]
            k = row.k
            lbl = L"K = %$k"
            cidx = findfirst(==(k), ks)
            c = colors[cidx]
            l = lines!(ax, t, log2.(abs_dρs), label=lbl,
                   color=c)
            translate!(l, (0,0,-100))
            i < nrow(subdf) || continue
            rowp1 = subdf[i+1,:]
            kp1 = rowp1.k
            p = n+1
            s = (kp1/k)^p
            ρs_kp1 = rowp1.data.ρ_err_norm
            ρ0_kp1 = ρs_kp1[1]
            abs_dρs_kp1 = [ abs(ρi[1] - ρ0_kp1[1]) for ρi in ρs_kp1 ]
            lines!(ax, t, log2.(s.*abs_dρs_kp1),
                   color=c, linestyle=:dash)
        end

        ylims!(ax, (-40,-3))
    end

    for i in 2:ncols
        hideydecorations!(axs[i], ticks=false, grid=false, minorticks=false, minorgrid=false)
    end
    linkxaxes!(axs...)
    linkyaxes!(axs...)

    _, idx_max_lbls = findmax(ax -> length(ax.scene.plots), axs)
    Legend(fig[0,1:ncols],
           axs[idx_max_lbls], colgap=40, orientation=:horizontal, width=nothing, tellwidth=false)
    Label(fig[1,0], text=L"\log_2(||\rho(t) - \rho(0)||_2)", rotation=pi/2,
          tellheight=false)

    for ax in axs
        ylims!(ax, (ymin,ymax))
        ax.yticks[] = yticks
    end

    rowgap!(fig.layout, 0)
    for i in 2:ncols
        colgap!(fig.layout, i, 0)
    end

    return fig

end


#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))_cvg"
display_plot = true

dirs = [ #"dg_n1_k8",
         "dg_n1_k16",
         "dg_n1_k32",
         "dg_n1_k64",
         "dg_n3_k8",
         "dg_n3_k16",
         "dg_n3_k32",
         "dg_n3_k64",
         "dg_n5_k8",
         "dg_n5_k16",
         "dg_n5_k32",
         "dg_n5_k64",
         "dg_n7_k8",
         "dg_n7_k16",
         "dg_n7_k32",
         "dg_n7_k64",
        ]
fig = with_theme(theme; fontsize=30) do
    make_plot(rootdir, dirs)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix).png"), fig)

end
