using CairoMakie
using ColorSchemes
using TOML
using Printf


include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot(rootdir::String, dir::String, tmax::Float64, n_lines::Int64)

    ### load
    data = readdata(joinpath(rootdir, dir), ["D","Sr","vr","smoothed_mu","γrr"],
                    output="output1d.h5")
    pars = TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml"))
    n = pars["Mesh"]["n"]
    k = pars["Mesh"]["k"]

    idx_tmax = findfirst(>(tmax), data.t) |> something
    dn = idx_tmax÷(n_lines-1)
    idxs = 1:dn:idx_tmax
    display(idxs)
    ts = data.t[idxs]
    x  = data.x
    x_bdrys = x[findall(i -> x[i]≈x[i+1], 1:length(x)-1)]
    Ds  = [ data.D[i] for i in idxs ]
    Srs = [ data.Sr[i] for i in idxs ]
    vrs = [ data.vr[i] for i in idxs ]
    μs  = [ data.smoothed_mu[i] for i in idxs ]

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1200,700))
    gl = fig[1,1] = GridLayout()
    gl_D  = gl[1,1] = GridLayout()
    gl_Sr = gl[2,1] = GridLayout()
    gl_vr = gl[3,1] = GridLayout()
    gl_μ  = gl[4,1] = GridLayout()
    gl_lbls = fig[1,0] = GridLayout()

    ax_D  = Axis(gl_D[1,1], xlabel=L"r")
    ax_Sr = Axis(gl_Sr[1,1], xlabel=L"r")
    ax_vr = Axis(gl_vr[1,1], xlabel=L"r")
    ax_μ  = Axis(gl_μ[1,1], xlabel=L"r")
    Label(gl_lbls[1,1], L"D(r)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[2,1], L"S_r(r)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[3,1], L"v_r(r)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[4,1], L"\mu(r)",
          tellwidth=true, tellheight=false, rotation=pi/2)

    colors = resample_cmap(ColorSchemes.turbo, length(ts))
    for (i,t) in enumerate(ts)
        str_t = @sprintf "%0.1f" t
        lbl = L"t = %$str_t"
        lines!(ax_D, x, Ds[i],
               color=colors[i])
        lines!(ax_Sr, x, Srs[i],
               color=colors[i])
        lines!(ax_vr, x, vrs[i],
               color=colors[i])
        lines!(ax_μ, x, μs[i],
               color=colors[i])
    end
    vlines!(ax_D, x_bdrys, color=:grey, linestyle=:dash)
    vlines!(ax_Sr, x_bdrys, color=:grey, linestyle=:dash)
    vlines!(ax_vr, x_bdrys, color=:grey, linestyle=:dash)
    vlines!(ax_μ, x_bdrys, color=:grey, linestyle=:dash)

    # tts = stack(ts for _ in 1:length(x))
    # h = heatmap!(ax_D, x, ts, tts', colormap=:turbo, visible=false)

    xlims!(ax_D, (12,18))
    xlims!(ax_Sr, (12,18))
    xlims!(ax_vr, (12,18))
    xlims!(ax_μ, (12,18))
    hidexdecorations!(ax_D, ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(ax_Sr, ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(ax_vr, ticks=false, grid=false, minorgrid=false, minorticks=false)
    rowgap!(gl, 0)


    ylims!(ax_D, (-0.005,0.025))
    ax_D.yticks[] = -0.02:0.01:0.02
    ylims!(ax_Sr, (-0.005,0.001))
    ax_Sr.yticks[] = -0.004:0.002:0.02
    ylims!(ax_vr, (-0.25,0.05))
    ax_vr.yticks[] = -0.20:0.1:0.1
    ylims!(ax_μ, (-0.007,0.067))
    ax_μ.yticks[] = 0.0:0.02:0.06

    cb = Colorbar(fig[0,1], limits=(0,round(tmax,digits=2)), nsteps=100,
                  colormap=colors,
                  vertical=false, alignmode=Outside(),
                  flipaxis=false, size=24, width=Relative(0.9), tickalign=1, ticksize=24)
    Label(fig[-1,1], L"t", tellwidth=false)
    cb.ticks[] = round.(ts, digits=2)

    return fig

end

#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))"
display_plot = true

tmax, n_ts = 0.4, 10
dir = "dg_n7_k16_first_steps"
fig = with_theme(theme; fontsize=28) do
    make_plot(rootdir, dir, tmax, n_ts)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_first_steps.png"), fig, backend=CairoMakie)

end
