using CairoMakie
using TOML
using Printf


include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot(rootdir::String, dirs::Vector{String}, ts::Vector{Float64})

    ### load
    data = [ readdata(joinpath(rootdir, dir), ["D","Sr","vr","smoothed_mu","γrr"],
                      output="output1d.h5") for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml"))
             for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    ### sort wrt k
    P          = sortperm(ks)
    data       = data[P]
    pars       = pars[P]
    ns         = ns[P]
    ks         = ks[P]
    dirs       = dirs[P]

    W(vr,γrr) = 1/sqrt(1 - vr^2/γrr)

    idxs_ts = [ [ something(findfirst(t -> t≈ti, d.t)) for ti in ts ]
               for d in data ]
    xs  = [ d.x for d in data ]
    Ds  = [ [ d.D[i] for i in idxs ]                for (idxs,d) in zip(idxs_ts,data) ]
    Srs = [ [ d.Sr[i] for i in idxs ]               for (idxs,d) in zip(idxs_ts,data) ]
    vrs = [ [ d.vr[i] for i in idxs ]               for (idxs,d) in zip(idxs_ts,data) ]
    Ws  = [ [ W.(d.vr[i],d.γrr[i]) for i in idxs ]  for (idxs,d) in zip(idxs_ts,data) ]
    μs  = [ [ d.smoothed_mu[i] for i in idxs ]      for (idxs,d) in zip(idxs_ts,data) ]

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1200,1200))
    gls = [ fig[1,i] = GridLayout() for i in 1:length(dirs) ]
    gls_D  = [ gl[1,1] = GridLayout() for gl in gls ]
    gls_Sr = [ gl[2,1] = GridLayout() for gl in gls ]
    gls_vr = [ gl[3,1] = GridLayout() for gl in gls ]
    gls_W  = [ gl[4,1] = GridLayout() for gl in gls ]
    gls_μ  = [ gl[5,1] = GridLayout() for gl in gls ]
    gl_lbls = fig[1,0] = GridLayout()

    axs_D  = [ Axis(gl[1,1], xlabel=L"r") for gl in gls_D ]
    axs_Sr = [ Axis(gl[1,1], xlabel=L"r") for gl in gls_Sr ]
    axs_vr = [ Axis(gl[1,1], xlabel=L"r") for gl in gls_vr ]
    axs_W  = [ Axis(gl[1,1], xlabel=L"r") for gl in gls_W ]
    axs_μ  = [ Axis(gl[1,1], xlabel=L"r") for gl in gls_μ ]
    Label(gl_lbls[1,1], L"D(r)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[2,1], L"S_r(r)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[3,1], L"v_r(r)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[4,1], L"W(r)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[5,1], L"\mu(r)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    for (j,_) in enumerate(dirs)
        k = ks[j]
        Box(gls[j][0,1], color=:white)
        Label(gls[j][0,1], text=L"K = %$k", tellheight=true, tellwidth=false)
    end

    for (j,_) in enumerate(dirs)
        for (i,t) in enumerate(ts)
            str_t = @sprintf "%0.1f" t
            lbl = L"t = %$str_t"
            lines!(axs_D[j], xs[j], Ds[j][i], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
            lines!(axs_Sr[j], xs[j], Srs[j][i], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
            lines!(axs_vr[j], xs[j], vrs[j][i], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
            lines!(axs_W[j], xs[j], Ws[j][i], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
            lines!(axs_μ[j], xs[j], μs[j][i], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
        end
    end

    for (j,_) in enumerate(dirs)
        for axs in (axs_D,axs_Sr,axs_vr,axs_W)
            for ax in axs
                hidexdecorations!(ax, ticks=false, grid=false, minorgrid=false, minorticks=false)
            end
        end
    end
    for axs in (axs_D,axs_Sr,axs_vr,axs_W,axs_μ)
        for (i,ax) in enumerate(axs)
            i == 1 && continue
            hideydecorations!(ax, ticks=false, grid=false, minorgrid=false, minorticks=false)
        end
    end

    linkxaxes!(axs_D...)
    linkxaxes!(axs_Sr...)
    linkxaxes!(axs_vr...)
    linkxaxes!(axs_W...)
    linkxaxes!(axs_μ...)
    Legend(fig.layout[0,1:end], first(axs_D), colgap=10, orientation=:horizontal, width=nothing, tellwidth=false,
           nbanks=1)
    rowgap!(fig.layout, 0)
    for gl in gls
        rowgap!(gl, 0)
    end
    rowgap!(gl_lbls, 0)

    for ax in axs_D
        ylims!(ax, (-0.05,0.47))
        ax.yticks[] = 0.0:0.1:0.4
    end
    for ax in axs_Sr
        ylims!(ax, (-0.22,0.02))
        ax.yticks[] = -0.2:0.05:0.0
    end
    for ax in axs_vr
        ylims!(ax, (-0.65,0.05))
        ax.yticks[] = -0.6:0.2:0.0
    end
    for ax in axs_W
        ylims!(ax, (0.99,1.14))
        ax.yticks[] = 1.0:0.04:1.14
    end
    for ax in axs_μ
        ylims!(ax, (-0.007,0.067))
        ax.yticks[] = 0.0:0.02:0.06
    end

    return fig

end

#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))"
display_plot = true

ts = [ 0.0, 10.0, 20.0, 30.0, 40.0, 100.0 ]
dirs = [
         "dg_n7_k8",
         "dg_n7_k16",
         "dg_n7_k32",
         "dg_n7_k64",
         ]
fig = with_theme(theme; fontsize=28) do
    make_plot(rootdir, dirs, ts)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_profiles.png"), fig, backend=CairoMakie)

end
