using CairoMakie
using CairoMakie.Makie.Colors


include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot(rootdir::String, dirs::Vector{String})

    for dir in dirs
        @assert isdir(joinpath(rootdir,dir)) "missing output directory $(joinpath(rootdir,dir))"
    end

    ### load
    data = [ readdata(joinpath(rootdir, dir), ["rho","smoothed_mu"],
                      output="output1d.h5") for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml")) for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    ### sort wrt k
    P          = sortperm(ks)
    data       = data[P]
    pars       = pars[P]
    ns         = ns[P]
    ks         = ks[P]
    dirs       = dirs[P]

    ts = [ d.t for d in data ]
    xs = [ d.x for d in data ]
    us = [ d.rho for d in data ]
    μs = [ d.smoothed_mu for d in data ]

    stack_μs   = stack.(μs)
    @show minimum(minimum, stack_μs)
    @show maximum(maximum, stack_μs)
    min_μ, max_μ = 0.0, 0.004
    Δμ = max_μ-min_μ
    yticks_μ = range(min_μ, max_μ, length=6)

    xmin, xmax = -0.25, 1.25

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1200,1000))
    gl_u = fig[1,1:2] = GridLayout()
    gl_μs = fig[2,1:2] = GridLayout()
    # gl_cb = fig[2,3] = GridLayout(; alignmode=Outside())

    ax_u = Axis(gl_u[1,1], xlabel=L"x")
    ax_μs = [ Axis(gl_μs[1,i], xlabel=L"x") for i in 1:length(dirs) ]

    # plot density
    ylabel_u=L"u(x)"
    Label(fig[1,0], ylabel_u,
          tellwidth=true, tellheight=false, rotation=pi/2, halign=:left)
    for (i,dir) in enumerate(dirs)
        lbl = "K = $(ks[i])"
        for ax in (ax_u,)
            lines!(ax, xs[i], us[i][end #= final time =#], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
        end
    end

    xlims!(ax_u, (xmin,xmax))
    # ylims!(ax_u, (-1.0,1.0))
    # ax_u.xticks[] = range(0.0,1.0,step=0.25)
    # ax_u.yticks[] = range(1.0,1.75,step=0.2)

    # plot the heatmaps for the viscosity
    ylabel_μ=L"\mu(t,x)"
    Label(fig[4,1:2], ylabel_μ,
          tellwidth=false, tellheight=true)
    Label(fig[2,0], L"t",
          tellwidth=true, tellheight=false, rotation=pi/2)
    heatmaps = []
    for (i,dir) in enumerate(dirs)
        h = heatmap!(ax_μs[i], xs[i], ts[i], stack_μs[i], colormap=:turbo,
                     colorrange=(min_μ,max_μ))
        translate!(h, 0, 0, -100)
        push!(heatmaps, h)
    end
    white = RGBA(1,1,1,1)
    for ax in ax_μs
        xlims!(ax, (xmin,xmax))
        ax.xtickcolor[] = white
        ax.ytickcolor[] = white
        ax.xminortickcolor[] = white
        ax.yminortickcolor[] = white
        ax.xgridcolor[] = white
        ax.ygridcolor[] = white
        ax.xminorgridcolor[] = white
        ax.yminorgridcolor[] = white
    end
    for ax in ax_μs[2:end]
        hideydecorations!(ax, ticks=false, grid=false, minorticks=false, minorgrid=false)
        linkaxes!(ax_μs[1], ax)
    end

    cb = Colorbar(fig[3,1:2], heatmaps[end], vertical=false, alignmode=Outside(),
                  flipaxis=false, size=24, width=Relative(0.9))
    cb.ticks[] = yticks_μ
    Legend(gl_u[0,1], ax_u, colgap=40, orientation=:horizontal, width=nothing, tellwidth=false)
    rowgap!(gl_u, 1, 0)

    return fig

end

#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))"
display_plot = true

dirs = [
         "mono1_avmda",
         "mono2_avmda",
         "mono3_avmda",
         ]
fig = with_theme(theme; fontsize=34) do
    make_plot(rootdir, dirs)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_mono_avmda.png"), fig)

end
