using CairoMakie
using CairoMakie.Makie.Colors # RGBA
using TOML
using Printf
using Roots

include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))

function make_plot(rootdir::String, dirs::Vector{String}, ts::Vector{Float64})

    for dir in dirs
        @assert isdir(joinpath(rootdir,dir)) "missing output directory $(joinpath(rootdir,dir))"
    end

    ### load
    data = [ readdata(joinpath(rootdir, dir), ["v"],
                      output="output1d.h5") for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(dir).toml")) for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    ### sort wrt k
    P          = sortperm(ks)
    data       = data[P]
    pars       = pars[P]
    ns         = ns[P]
    ks         = ks[P]
    dirs       = dirs[P]

    idxs_ts = [ [ something(findfirst(ti -> ti≈t, d.t)) for t in ts ] for d in data ]
    xs = [ d.x for d in data ]
    vs = [ [ d.v[i] for i in idxs ] for (idxs,d) in zip(idxs_ts,data) ]

    ref_vs = [ [ simple_wave_analytical(x,t) for t in ts ] for x in xs ]

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1900,600))
    nrows, ncols = 2, 2
    @assert length(ts) == nrows*ncols
    gls_v = [ fig[i,j] = GridLayout() for i in 1:nrows, j in 1:ncols ]
    axs_v = [ Axis(gls_v[i,j][1,1], xlabel=L"x") for i in 1:nrows, j in 1:ncols ]

    for (t,gl) in zip(ts,gls_v)
        gg = GridLayout(gl[1,1], tellwidth=false, tellheight=false,
                        halign=:center, valign=:top)
        box = Box(gg[1,1], color=:white, strokecolor=:black)
        str_t = @sprintf "%.2f" t
        lbl = Label(gg[1,1], text=L"t = %$t", padding=(30,30,10,10))
        translate!(box.blockscene, (0,0,100))
        translate!(lbl.blockscene, (0,0,100))
    end

    @assert allequal(ns)
    n = ns[1]
    for (it,t) in enumerate(ts)
        ax = axs_v[it]
        for (ik,k) in enumerate(ks)
            ref_v = ref_vs[ik][it]
            err = ref_v .- vs[ik][it]
            lbl = L"K = %$k"
            lines!(ax, xs[ik], log2.(abs.(err)), label=lbl,
                   color=colors[periodic_index(ik,length(colors))])
            if ik > 1
                km1 = ks[ik-1]
                s = (k/km1)^(n)
                lines!(ax, xs[ik], log2.(s*abs.(err)),
                       color=(colors[periodic_index(ik-1,length(colors))],0.5),
                       linestyle=:dash)
            end
        end
    end

    for i in 1:nrows, j in 2:ncols
        hideydecorations!(axs_v[i,j], ticks=false, grid=false, minorticks=false, minorgrid=false)
    end
    for i in 1:nrows-1, j in 1:ncols
        hidexdecorations!(axs_v[i,j], ticks=false, grid=false, minorticks=false, minorgrid=false)
    end
    linkxaxes!(axs_v...)
    linkyaxes!(axs_v...)

    Legend(fig[0,1:ncols],
           first(axs_v), colgap=40, orientation=:horizontal, width=nothing, tellwidth=false)
    Label(fig[1:nrows,0], text=L"\log_2(|\epsilon|)", rotation=pi/2,
          tellheight=false)

    for ax in axs_v
        ax.xticks[] = -1.5:0.5:1.5
    end
    for ax in axs_v
        ylims!(ax, (-55,12))
    end

    rowgap!(fig.layout, 0)
    colgap!(fig.layout, 0)


    return fig

end


function simple_wave_analytical(xs, t)
    # Use the analytical solution from Liang 1977.
    # See also Anile 1989, chapter 5 for derivation of the analytical solution.
    # Note that the Anile book does not derive the Riemann invariant for the polytropic EoS
    #
    # The exact implicit solution is given in Liang, (II.12a) and (II.15).
    # To obtain the solution simply solve the following equation for v
    #   v = f(ξ(v))
    #   ξ(v) = x - (v+c_s(v))/(1+v c_s(v)) t
    # and c_s(v) is given by (II.15).

    # EoS parameters to match ID in dg1d.jl
    # those parameters were chosen to match the ones used Bernuzzi, Dietrich 2016 - WENO
    # same parameters were also used in Doulis, et al 2022, and Doulis, et al 2024
    Γ, K = 5/3, 100.0 # Γ ≡ γ in paper
    a = sqrt(Γ-1)
    d = a/2
    X = 0.3
    # velocity perturbation profile
    f(x) = (abs(x)<=X) * 0.5 * sin(pi/2*(x/X-1))^6

    # reference solution
    ρref = 1.0
    vref = 0.0
    pref = K*ρref^Γ
    ϵref = K / (Γ-1) * ρref^(Γ-1)
    href = 1 + ϵref + pref/ρref
    χ = K*Γ*ρref^(Γ-1)
    κ = 0
    cs0  = sqrt((χ + pref/ρref^2 * κ)/href)
    # cs0 = sqrt(K*Γ*ρref^(Γ-1)/(1+(K+K/(Γ-1))*ρref^(Γ-1)))
    # @show cs0, cs0_v2
    b = (a+cs0)/(a-cs0)

    vs = similar(xs)
    for (i,x) in enumerate(xs)
        c_s(v) = a * (b*((1+v)/(1-v))^d - 1) / (b*((1+v)/(1-v))^d + 1)
        ξ(v) = x - (v+c_s(v))/(1+v*c_s(v))*t
        F(v) = v - f(ξ(v))
        v = try
            find_zero(F, cs0, Order1())
        catch
            # the solver eventually fails, because the solution is only valid till
            # characteristics merge
            NaN
        end
        vs[i] = v
    end

    # linearly interpolate the NaN data points between their neighbors
    idxs = findall(isnan, vs)
    if length(idxs) > 1
        for i in idxs
            @assert 2 ≤ i ≤ length(xs)-1
            il, ir = i-1, i+1
            while il >= 2
                !isnan(vs[il]) && break
                il -= 1
            end
            while ir >= 2
                !isnan(vs[ir]) && break
                ir += 1
            end
            x = xs[i]
            xl, xr = xs[il], xs[ir]
            vl, vr = vs[il], vs[ir]
            @assert !isnan(vl)
            @assert !isnan(vr)
            v = (x-xl)/(xr-xl) * (vr-vl) + vl
            vs[i] = v
        end
    end

    return vs
end


function make_plot_selfcvg(rootdir::String, dirs::Vector{String}, ts::Vector{Float64}, var::String)

    for dir in dirs
        @assert isdir(joinpath(rootdir,dir)) "missing output directory $(joinpath(rootdir,dir))"
    end

    ### load
    data = [ readdata(joinpath(rootdir, dir), [var],
                      output="interpolate.h5") for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(dir).toml")) for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    ### sort wrt k
    P          = sortperm(ks)
    data       = data[P]
    pars       = pars[P]
    ns         = ns[P]
    ks         = ks[P]
    dirs       = dirs[P]

    idxs_ts = [ [ something(findfirst(ti -> ti≈t, d.t)) for t in ts ] for d in data ]
    xs = [ d.x for d in data ]
    vars = [ [ getproperty(d, Symbol(var))[i] for i in idxs ] for (idxs,d) in zip(idxs_ts,data) ]

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1900,600))
    nrows, ncols = 2, 2
    @assert length(ts) == nrows*ncols
    gls = [ fig[i,j] = GridLayout() for i in 1:nrows, j in 1:ncols ]
    axs = [ Axis(gls[i,j][1,1], xlabel=L"x") for i in 1:nrows, j in 1:ncols ]

    for (t,gl) in zip(ts,gls)
        gg = GridLayout(gl[1,1], tellwidth=false, tellheight=false,
                        halign=:center, valign=:top)
        box = Box(gg[1,1], color=:white, strokecolor=:black)
        str_t = @sprintf "%.2f" t
        lbl = Label(gg[1,1], text=L"t = %$t", padding=(30,30,10,10))
        translate!(box.blockscene, (0,0,100))
        translate!(lbl.blockscene, (0,0,100))
    end

    @assert allequal(ns)
    n = ns[1]
    for (it,t) in enumerate(ts)
        ax = axs[it]
        for (ik,k) in enumerate(ks[1:end-1])
            ik1, ik2 = ik, ik+1
            k1, k2 = ks[ik1], ks[ik2]
            var1, var2 = vars[ik1][it], vars[ik2][it]
            err = var1 .- var2
            lbl = L"\Delta K = %$k1 - %$k2"
            lines!(ax, xs[ik], log2.(abs.(err)), label=lbl,
                   color=colors[periodic_index(ik,length(colors))])
            if ik1 > 1
                s = (k2/k1)^(n)
                lines!(ax, xs[ik], log2.(s*abs.(err)),
                       color=(colors[periodic_index(ik,length(colors))],0.5),
                       linestyle=:dash)
            end
        end
    end

    for i in 1:nrows, j in 2:ncols
        hideydecorations!(axs[i,j], ticks=false, grid=false, minorticks=false, minorgrid=false)
    end
    for i in 1:nrows-1, j in 1:ncols
        hidexdecorations!(axs[i,j], ticks=false, grid=false, minorticks=false, minorgrid=false)
    end
    linkxaxes!(axs...)
    linkyaxes!(axs...)

    Legend(fig[0,1:ncols],
           first(axs), colgap=40, orientation=:horizontal, width=nothing, tellwidth=false)
    Label(fig[1:nrows,0], text=L"\log_2(|\epsilon|)", rotation=pi/2,
          tellheight=false)

    for ax in axs
        ax.xticks[] = -1.5:0.5:1.5
    end
    for ax in axs
        ylims!(ax, (-55,12))
    end

    rowgap!(fig.layout, 0)
    colgap!(fig.layout, 0)

    return fig
end

#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))"
display_plot = true

ts = [ 0.6, 0.75, 0.96, 1.2 ]
# ref_xs = collect(range(-0.25,1.25,length=500))
# ref_vs = [ simple_wave_analytical(ref_xs, t) for t in ts ]

dirs = [ "cov_avmda_n7_k40_cvgtest",
         "cov_avmda_n7_k80_cvgtest",
         "cov_avmda_n7_k160_cvgtest",
        ]
fig = with_theme(theme; fontsize=30) do
    make_plot(rootdir, dirs, ts)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_cov_avmda_n7_cvgtest.png"), fig)

dirs = [ "cov_avmda_n5_k40_cvgtest",
         "cov_avmda_n5_k80_cvgtest",
         "cov_avmda_n5_k160_cvgtest",
        ]

fig = with_theme(theme; fontsize=30) do
    make_plot(rootdir, dirs, ts)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_cov_avmda_n5_cvgtest.png"), fig)

dirs = [ "cov_avmda_n9_k40_cvgtest",
         "cov_avmda_n9_k80_cvgtest",
         "cov_avmda_n9_k160_cvgtest",
        ]

fig = with_theme(theme; fontsize=30) do
    make_plot(rootdir, dirs, ts)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_cov_avmda_n9_cvgtest.png"), fig)


vars = [ "D", "v" ]
for var in vars

    dirs = [ "cov_avmda_n7_k40_cvgtest",
             "cov_avmda_n7_k80_cvgtest",
             "cov_avmda_n7_k160_cvgtest",
            ]
    fig = with_theme(theme; fontsize=30) do
        make_plot_selfcvg(rootdir, dirs, ts, var)
    end
    display_plot && display(fig)
    save(joinpath(rootdir, "$(plotname_prefix)_cov_avmda_n7_cvgtest_selfcvg_$var.png"), fig)

    dirs = [ "cov_avmda_n9_k40_cvgtest",
             "cov_avmda_n9_k80_cvgtest",
             "cov_avmda_n9_k160_cvgtest",
            ]
    fig = with_theme(theme; fontsize=30) do
        make_plot_selfcvg(rootdir, dirs, ts, var)
    end
    display_plot && display(fig)
    save(joinpath(rootdir, "$(plotname_prefix)_cov_avmda_n9_cvgtest_selfcvg_$var.png"), fig)

    dirs = [ "cov_avmda_n5_k40_cvgtest",
             "cov_avmda_n5_k80_cvgtest",
             "cov_avmda_n5_k160_cvgtest",
            ]
    fig = with_theme(theme; fontsize=30) do
        make_plot_selfcvg(rootdir, dirs, ts, var)
    end
    display_plot && display(fig)
    save(joinpath(rootdir, "$(plotname_prefix)_cov_avmda_n5_cvgtest_selfcvg_$var.png"), fig)

end

end
