using CairoMakie
using CairoMakie.Makie.Colors # RGBA
using TOML
using Printf
using Roots

include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot(rootdir::String, dir::String, ts::Vector{Float64},
        ref_x::Vector{Float64}, ref_vs::Vector{Vector{Float64}})

    data = readdata(joinpath(rootdir, dir), ["smoothed_mu","v"], output="output1d.h5")
    pars = TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml"))

    idxs = [ something(findfirst(t -> t≈ti, data.t)) for ti in ts ]
    xs = data.x
    vs = [ data.v[i] for i in idxs ]
    μs = [ data.smoothed_mu[i] for i in idxs ]

    idxs_bdry = findall(i -> xs[i]≈xs[i+1], 1:length(xs)-1)
    pushfirst!(idxs_bdry, 1)
    push!(idxs_bdry, length(xs))
    xs_bdry = xs[idxs_bdry]

    stack_μs = stack(data.smoothed_mu)

    ### axis limits
    xmin, xmax = -0.3, 1.3
    vmin, vmax = -0.05, 0.55
    tmin, tmax = 0.65, 1.2
    μmin, μmax = 0.0, 0.0016
    ### axis ticks
    x_ticks = -0.25:0.25:1.25
    v_ticks = 0.0:0.1:0.5
    t_ticks = 0.0:0.1:1.2
    μ_ticks = range(μmin,μmax,length=6)

    # idx_tmin = findfirst(>=(tmin), data.t)
    # idx_tmax = findfirst(>=(tmax), data.t)
    # @assert idx_tmin > 1
    # @assert idx_tmax < length(data.t)
    # idx_tmin -= 1
    # idx_tmax += 1
    # subt = @view data.t[idx_tmin:idx_tmax]
    # subμ = @view stack_μs[:,idx_tmin:idx_tmax]
    # t_stripes = [ [subt[i],subt[i+1]] for i in 1:length(subt)-1 ]
    # μ_stripes = [ zeros(Float64,(length(xs),2)) for i in 1:length(subt)-1 ]
    # for i in 1:length(subt)-1
    #     @. μ_stripes[i][:,1] = subμ[:,i]
    #     @. μ_stripes[i][:,2] = subμ[:,i]
    # end

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1200,1000))
    gl_v = fig[1,1] = GridLayout()
    gl_μ = fig[2,1] = GridLayout()

    ax_v = Axis(gl_v[1,1], xlabel=L"x")
    ax_μ = Axis(gl_μ[1,1], xlabel=L"x")
    Label(fig[1,0], L"v_x(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(fig[2,0], L"t",
          tellwidth=true, tellheight=false, rotation=pi/2)

    for (i,t) in enumerate(ts)
        str_t = @sprintf "%0.2f" t
        lbl = L"t = %$str_t"
        lines!(ax_v, xs, vs[i], label=lbl,
               color=colors[periodic_index(i,length(colors))])
        lines!(ax_v, ref_x, ref_vs[i],
               color=(:black,1.0), linestyle=:dash)
    end
    h = heatmap!(ax_μ, xs, data.t, stack_μs, colormap=:turbo,
             colorrange=(μmin,μmax))
    # hs = Any[]
    # for (t_str,μ_str) in zip(t_stripes,μ_stripes)
    #     h = heatmap!(ax_μ, xs, t_str, μ_str,
    #                  colormap=:turbo, colorrange=(μmin,μmax))
    #     translate!(h, 0, 0, -100)
    #     push!(hs, h)
    # end
    vlines!(ax_μ, xs_bdry, color=:white, linestyle=:dash, linewidth=2)
    vlines!(ax_v, xs_bdry, color=(:black,0.3), linestyle=:dash, linewidth=2)
    translate!(h, 0, 0, -100)

    cb = Colorbar(fig[3,1], h, vertical=false, alignmode=Outside(),
    # cb = Colorbar(fig[3,1], first(hs), vertical=false, alignmode=Outside(),
                  flipaxis=false, size=24, width=Relative(0.9))
    Label(fig[4,1], L"\mu(t,x)", alignmode=Outside(),
          tellheight=true, tellwidth=false)
    Legend(fig.layout[0,1], ax_v, colgap=10, orientation=:horizontal, width=nothing, tellwidth=false,
           nbanks=1)
    rowgap!(fig.layout, 1, 0)

    # make the grid lines of the heatmap white
    white = RGBA(1,1,1,1)
    ax_μ.xtickcolor[] = white
    ax_μ.ytickcolor[] = white
    ax_μ.xminortickcolor[] = white
    ax_μ.yminortickcolor[] = white
    ax_μ.xgridcolor[] = white
    ax_μ.ygridcolor[] = white
    ax_μ.xminorgridcolor[] = white
    ax_μ.yminorgridcolor[] = white

    hidexdecorations!(ax_μ, ticks=false, minorticks=false, ticklabels=false)
    hidexdecorations!(ax_v, ticks=false, minorticks=false)

    # set limits and ticks
    linkxaxes!(ax_v, ax_μ)
    xlims!(ax_v, (xmin,xmax))
    ylims!(ax_v, (vmin,vmax))
    xlims!(ax_μ, (xmin,xmax))
    ylims!(ax_μ, (tmin,tmax))
    ax_v.xticks[] = x_ticks
    ax_v.yticks[] = v_ticks
    ax_μ.xticks[] = x_ticks
    ax_μ.yticks[] = t_ticks
    cb.ticks[] = μ_ticks

    return fig

end


function simple_wave_analytical(xs, t)
    # Use the analytical solution from Liang 1977.
    # See also Anile 1989, chapter 5 for derivation of the analytical solution.
    # Note that the Anile book does not derive the Riemann invariant for the polytropic EoS
    #
    # The exact implicit solution is given in Liang, (II.12a) and (II.15).
    # To obtain the solution simply solve the following equation for v
    #   v = f(ξ(v))
    #   ξ(v) = x - (v+c_s(v))/(1+v c_s(v)) t
    # and c_s(v) is given by (II.15).

    # EoS parameters to match ID in dg1d.jl
    # those parameters were chosen to match the ones used Bernuzzi, Dietrich 2016 - WENO
    # same parameters were also used in Doulis, et al 2022, and Doulis, et al 2024
    Γ, K = 5/3, 100.0 # Γ ≡ γ in paper
    a = sqrt(Γ-1)
    d = a/2
    X = 0.3
    # velocity perturbation profile
    f(x) = (abs(x)<=X) * 0.5 * sin(pi/2*(x/X-1))^6

    # reference solution
    ρref = 1.0
    vref = 0.0
    pref = K*ρref^Γ
    ϵref = K / (Γ-1) * ρref^(Γ-1)
    href = 1 + ϵref + pref/ρref
    χ = K*Γ*ρref^(Γ-1)
    κ = 0
    cs0  = sqrt((χ + pref/ρref^2 * κ)/href)
    # cs0 = sqrt(K*Γ*ρref^(Γ-1)/(1+(K+K/(Γ-1))*ρref^(Γ-1)))
    # @show cs0, cs0_v2
    b = (a+cs0)/(a-cs0)

    vs = similar(xs)
    for (i,x) in enumerate(xs)
        c_s(v) = a * (b*((1+v)/(1-v))^d - 1) / (b*((1+v)/(1-v))^d + 1)
        ξ(v) = x - (v+c_s(v))/(1+v*c_s(v))*t
        F(v) = v - f(ξ(v))
        v = try
            find_zero(F, cs0, Order1())
        catch
            # the solver eventually fails, because the solution is only valid till
            # characteristics merge
            NaN
        end
        vs[i] = v
    end

    # linearly interpolate the NaN data points between their neighbors
    idxs = findall(isnan, vs)
    if length(idxs) > 1
        for i in idxs
            @assert 2 ≤ i ≤ length(xs)-1
            il, ir = i-1, i+1
            while il >= 2
                !isnan(vs[il]) && break
                il -= 1
            end
            while ir >= 2
                !isnan(vs[ir]) && break
                ir += 1
            end
            x = xs[i]
            xl, xr = xs[il], xs[ir]
            vl, vr = vs[il], vs[ir]
            @assert !isnan(vl)
            @assert !isnan(vr)
            v = (x-xl)/(xr-xl) * (vr-vl) + vl
            vs[i] = v
        end
    end

    return vs
end

#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))_v_mu"
display_plot = true

ts = [ 0.0, 0.6, 0.75, 0.96, 1.2 ]
ref_xs = collect(range(-1.25,1.25,length=200))
ref_vs = [ simple_wave_analytical(ref_xs, t) for t in ts ]

dir = "mono_avmda_n7_k80"
fig = with_theme(theme; fontsize=36) do
    make_plot(rootdir, dir, ts, ref_xs, ref_vs)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_$(dir).png"), fig)

dir = "cov_avmda_n7_k80"
fig = with_theme(theme; fontsize=36) do
    make_plot(rootdir, dir, ts, ref_xs, ref_vs)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_$(dir).png"), fig)

dir = "drag_mono_avmda_n7_k80"
fig = with_theme(theme; fontsize=36) do
    make_plot(rootdir, dir, ts, ref_xs, ref_vs)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_$(dir).png"), fig)

dir = "drag_cov_avmda_n7_k80"
fig = with_theme(theme; fontsize=36) do
    make_plot(rootdir, dir, ts, ref_xs, ref_vs)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_$(dir).png"), fig)

end
