using HDF5
using LinearAlgebra
using CairoMakie
import CairoMakie: Makie
import CairoMakie.Makie.GeometryBasics: coordinates


function readdata(path::AbstractString, varnames::Vector; output="output.h5")
    @assert isdir(path) path
    outfile = joinpath(path, output)
    @assert isfile(outfile) outfile

    data = h5open(outfile, "r") do file
        groups = keys(file)
        filter!(!=("mesh"), groups)
        groups = tryparse.(Int, groups)
        sort!(groups)
        x = read(file["mesh"], "x")
        vars = Any[]
        t = Float64[]
        for (j,grp) in enumerate(groups)
            g = file["$grp"]
            for (i,name) in enumerate(varnames)
                if j == 1
                    push!(vars, [ read(g,name) ])
                else
                    push!(vars[i], read(g, name))
                end
            end
            push!(t, read(g, "t"))
        end
        nt = NamedTuple((Symbol(k),v) for (k,v) in zip(varnames,vars))
        nnt = merge(nt, (x = x, t = t,))
        nnt
    end
end


function findpeaks(data::AbstractVector{T}) where T<:Real
    ms, is = T[], Int64[]
    m, i = findmax(data)
    push!(ms, m)
    push!(is, i)
    # we identify a point as a peak, if the slope changes from up to down wrt to the
    # left and right neighbors
    N = length(data)
    for i = 2:N-1
        dl, dc, dr = data[i-1], data[i], data[i+1]
        if dc-dl > T(0) && dr-dc < T(0)
            push!(ms, dc)
            push!(is, i)
        end
    end
    unique!(ms)
    unique!(is)
    return ms, is
end


# logtrafo(v) = log10(abs(!isfinite(v) ? NaN : v))
function logtrafo(v)
    !isfinite(v) && return NaN
    vv = log10(abs(v))
    return isfinite(vv) ? vv : -20
end


function least_sqr_linear(x, y)
  n = length(x)
  @assert n == length(y) "Mismatch in number of data points x, y!"

  yx = dot(y, x)
  xx = dot(x, x)
  mean_y = sum(y) / n
  mean_x = sum(x) / n

  ss_xx = xx - mean_x^2 * n
  ss_xy = yx - mean_x * mean_y * n

  b = ss_xy / ss_xx
  a = mean_y - b * mean_x

  f(x) = a + b * x
  return f, a, b
end


function get_parameter(path, parameter, section)
    parfile = joinpath(path, "all_parameters.toml")
    prms = TOML.parsefile(parfile)
    sec = get(prms, section, nothing)
    isnothing(sec) && return nothing
    return get(sec, parameter, nothing)
end

periodic_index(i,N) = (i-1)%N+1

plotname(scriptname, extra) = string(first(splitext(scriptname)), "_", extra)


function get_inset_bbox(inset::Axis, ax::Axis)
    shifted_bbox1 = lift(ax.scene.viewport, inset.layoutobservables.computedbbox,
                        ax.scene.camera.projectionview) do viewport, bbox, _
      Rect2f(bbox.origin .- viewport.origin, bbox.widths)
    end
end


function vertices_inset_and_bbox(inset::Axis, bbox, ax::Axis)
    vs = [ Observable{Any}() for _ = 1:4 ]
    onany(inset.finallimits, bbox) do bb1, bb2
      for (i,(c1, c2)) in enumerate(zip(coordinates(bb1), coordinates(bb2)))
        cc1 = Makie.project(ax.scene, Point2f(c1))
        vs[i][] = [ cc1, Point2f(c2) ]
      end
    end
    notify(inset.finallimits)
    return vs
end


function place_inset_axis!(ax::Axis; kwargs...)
    place_inset_axis!(get_gridposition_in_parent(ax); kwargs...)
end

function place_inset_axis!(p::GridPosition;
        inset_width::Relative=Relative(0.4), inset_height::Relative=Relative(0.4),
        inset_halign::Float64=0.0, inset_valign::Float64=0.0, inset_alignmode=Outside(),
        ax_kwargs...)
    gl = p.layout[p.span.rows,p.span.cols] = GridLayout(; width=inset_width, height=inset_height,
                                                          halign=inset_halign, valign=inset_valign,
                                                          alignmode=inset_alignmode)
    ax = Axis(gl[1,1]; ax_kwargs...)
    translate!(ax.blockscene, 0, 0, 1000)
    return ax
end


function draw_inset_frame!(ax::Axis, inset::Axis;
        color=(:white,0.0), strokecolor=:black, strokewidth=1,
        vertex_color=(:grey,0.4))
    poly!(ax, inset.finallimits; color, strokecolor, strokewidth)
    bbox = get_inset_bbox(inset, ax)
    vs = vertices_inset_and_bbox(inset, bbox, ax)
    for v in vs
        l = lines!(ax, v, color=vertex_color, space=:pixel, linewidth=1)
    end
end


function get_gridposition_in_parent(ax::Axis)
    layout = ax.parent.layout
    for c in layout.content
        if c.content === ax
            return layout[c.span.rows,c.span.cols]
        end
    end
    error("upsi")
    nothing
end

# ax.parent.layout.content[idx].content === ax to find grid span in parent layout for axis
