using CairoMakie
using TOML
using Printf


include(joinpath(@__DIR__,"..","theme.jl"))
include(joinpath(@__DIR__,"..","utils.jl"))


function make_plot(rootdir::String, dirs::Vector{String}, refsol_dir::String, ts::Vector{Float64})

    ### load
    data = [ readdata(joinpath(rootdir, dir), ["rho","smoothed_mu","p","q"],
                      output="output1d.h5") for dir in dirs ]
    pars = [ TOML.parsefile(joinpath(rootdir,dir,"$(replace(dir,".previous"=>"")).toml"))
             for dir in dirs ]
    ns = [ par["Mesh"]["n"] for par in pars ]
    ks = [ par["Mesh"]["k"] for par in pars ]

    fv_data = readdata(joinpath(rootdir, refsol_dir), ["rho","p","q"], output="output1d.h5")

    ### sort wrt k
    P          = sortperm(ks)
    data       = data[P]
    pars       = pars[P]
    ns         = ns[P]
    ks         = ks[P]
    dirs       = dirs[P]

    idxs_ts = [ [ something(findfirst(t -> t≈ti, d.t)) for ti in ts ]
               for d in data ]
    xs = [ d.x for d in data ]
    ρs = [ [ d.rho[i] for i in idxs ]         for (idxs,d) in zip(idxs_ts,data) ]
    ps = [ [ d.p[i] for i in idxs ]           for (idxs,d) in zip(idxs_ts,data) ]
    vs = [ [ d.q[i]./d.rho[i] for i in idxs ] for (idxs,d) in zip(idxs_ts,data) ]
    μs = [ [ d.smoothed_mu[i] for i in idxs ] for (idxs,d) in zip(idxs_ts,data) ]

    fv_idxs_ts = [ something(findfirst(t -> t≈ti, fv_data.t)) for ti in ts ]
    fv_x  = fv_data.x
    fv_ρs = [ fv_data.rho[i]          for i in fv_idxs_ts ]
    fv_ps = [ fv_data.p[i]            for i in fv_idxs_ts ]
    fv_vs = [ fv_data.q[i]./fv_data.rho[i]  for i in fv_idxs_ts ]

    #######################################################################
    #                              plotting                               #
    #######################################################################

    fig = Figure(size=(1200,1000))
    gls = [ fig[1,i] = GridLayout() for i in 1:length(dirs) ]
    gls_ρ = [ gl[1,1] = GridLayout() for gl in gls ]
    gls_p = [ gl[2,1] = GridLayout() for gl in gls ]
    gls_v = [ gl[3,1] = GridLayout() for gl in gls ]
    gls_μ = [ gl[4,1] = GridLayout() for gl in gls ]
    gl_lbls = fig[1,0] = GridLayout()

    axs_ρ = [ Axis(gl[1,1], xlabel=L"x") for gl in gls_ρ ]
    axs_p = [ Axis(gl[1,1], xlabel=L"x") for gl in gls_p ]
    axs_v = [ Axis(gl[1,1], xlabel=L"x") for gl in gls_v ]
    axs_μ = [ Axis(gl[1,1], xlabel=L"x") for gl in gls_μ ]
    Label(gl_lbls[1,1], L"\rho(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[2,1], L"p(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[3,1], L"v(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    Label(gl_lbls[4,1], L"\mu(x)",
          tellwidth=true, tellheight=false, rotation=pi/2)
    for (j,_) in enumerate(dirs)
        k = ks[j]
        Box(gls[j][0,1], color=:white)
        Label(gls[j][0,1], text=L"K = %$k", tellheight=true, tellwidth=false)
    end

    for (j,_) in enumerate(dirs)
        for (i,t) in enumerate(ts)
            str_t = @sprintf "%0.3f" t
            lbl = L"t = %$str_t"
            lines!(axs_ρ[j], fv_x, fv_ρs[i],
                   color=(colors[periodic_index(i,length(colors))],0.5),
                   linestyle=:dash)
            lines!(axs_ρ[j], xs[j], ρs[j][i], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
            lines!(axs_p[j], fv_x, fv_ps[i],
                   color=(colors[periodic_index(i,length(colors))],0.5),
                   linestyle=:dash)
            lines!(axs_p[j], xs[j], ps[j][i], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
            lines!(axs_v[j], fv_x, fv_vs[i],
                   color=(colors[periodic_index(i,length(colors))],0.5),
                   linestyle=:dash)
            lines!(axs_v[j], xs[j], vs[j][i], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
            lines!(axs_μ[j], xs[j], μs[j][i], label=lbl,
                   color=colors[periodic_index(i,length(colors))])
        end
    end

    for (j,_) in enumerate(dirs)
        for axs in (axs_ρ,axs_p,axs_v)
            for ax in axs
                hidexdecorations!(ax, ticks=false, grid=false, minorgrid=false, minorticks=false)
            end
        end
    end
    for axs in (axs_ρ,axs_p,axs_v,axs_μ)
        for (i,ax) in enumerate(axs)
            i == 1 && continue
            hideydecorations!(ax, ticks=false, grid=false, minorgrid=false, minorticks=false)
        end
    end


    linkxaxes!(axs_ρ...)
    linkxaxes!(axs_p...)
    linkxaxes!(axs_v...)
    linkxaxes!(axs_μ...)
    Legend(fig.layout[0,1:end], first(axs_ρ), colgap=10, orientation=:horizontal, width=nothing, tellwidth=false,
           nbanks=1)
    rowgap!(fig.layout, 0)
    for gl in gls
        rowgap!(gl, 0)
    end
    rowgap!(gl_lbls, 0)

    for ax in axs_ρ
        ylims!(ax, (0.3,6.8))
        ax.yticks[] = 1.0:1.0:6.0
    end
    for ax in axs_p
        ylims!(ax, (-50.0,1050.0))
        ax.yticks[] = 0.0:250.0:1000.0
    end
    for ax in axs_v
        ylims!(ax, (-2.0,22.0))
        ax.yticks[] = 0.0:5.0:20.0
    end
    for ax in axs_μ
        ylims!(ax, (-0.01, 0.07))
        ax.yticks[] = 0.00:0.02:0.1
    end

    return fig

end

#######################################################################
#                             parameters                              #
#######################################################################

let

rootdir = @__DIR__
plotname_prefix = "$(basename(rootdir))_rho_mu"
display_plot = true

ts = [ 0.0, 0.003, 0.006, 0.009, 0.012 ]
dirs = [
         "mono1_avmda",
         "mono2_avmda",
         "mono3_avmda",
         # "mono4_avmda",
         ]
refsol_dir = "ref_fv"
fig = with_theme(theme; fontsize=28) do
    make_plot(rootdir, dirs, refsol_dir, ts)
end
display_plot && display(fig)
save(joinpath(rootdir, "$(plotname_prefix)_evo_combined.png"), fig, backend=CairoMakie)

end
